#ifndef LEU_LIB_H
#define LEU_LIB_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
  typedef __int8            int8_t;
  typedef __int16           int16_t;
  typedef __int32           int32_t;
  typedef __int64           int64_t;
  typedef unsigned __int8   uint8_t;
  typedef unsigned __int16  uint16_t;
  typedef unsigned __int32  uint32_t;
  typedef unsigned __int64  uint64_t;
#else
  #include <stdint.h>
#endif

typedef int32_t LEU;

////////////////////////////////////////////////////////////
// Return code
////////////////////////////////////////////////////////////
#define LEU_FAIL		-1
#define LEU_SUCCESS		0

////////////////////////////////////////////////////////////
// Error
////////////////////////////////////////////////////////////
#define LEU_NOERROR				0
#define LEU_ERROR_NOINIT		1
#define LEU_ERROR_NOCONNECT		2
#define LEU_ERROR_NOMEASURE		3
#define LEU_ERROR_MEASURING		4
#define LEU_ERROR_PARAMETER		5
#define LEU_ERROR_NOSERIAL		6
#define LEU_ERROR_COM			7
#define LEU_ERROR_ALLOC			8
#define LEU_ERROR_TIMEOUT		9
#define LEU_ERROR_VERSION		10
#define LEU_ERROR_NODATA		11

////////////////////////////////////////////////////////////
// State
////////////////////////////////////////////////////////////
#define LEU_STATE_STOP			0
#define LEU_STATE_MEASURING		1
#define LEU_STATE_OVERFLOW		2

////////////////////////////////////////////////////////////
// VBUS
////////////////////////////////////////////////////////////
#define LEU_VBUS_CYCLE_100ms    0
#define LEU_VBUS_CYCLE_200ms    1
#define LEU_VBUS_CYCLE_500ms    2
#define LEU_VBUS_CYCLE_1s       3
#define LEU_VBUS_CYCLE_0_1ms    4
#define LEU_VBUS_CYCLE_0_2ms    5
#define LEU_VBUS_CYCLE_0_5ms    6
#define LEU_VBUS_CYCLE_1ms      7
#define LEU_VBUS_CYCLE_2ms      8
#define LEU_VBUS_CYCLE_5ms      9
#define LEU_VBUS_CYCLE_10ms     10
#define LEU_VBUS_CYCLE_20ms     11
#define LEU_VBUS_CYCLE_50ms     12

////////////////////////////////////////////////////////////
// Filter
////////////////////////////////////////////////////////////
#define LEU_FILTER_IN_NAK     0x0010
#define LEU_FILTER_OUT_NAK    0x0020
#define LEU_FILTER_SETUP_NAK  0x0040
#define LEU_FILTER_PING       0x0080
#define LEU_FILTER_SOF        0x0100

#define LEU_FILTER_00         0x00000001
#define LEU_FILTER_IN01       0x00000002
#define LEU_FILTER_IN02       0x00000004
#define LEU_FILTER_IN03       0x00000008
#define LEU_FILTER_IN04       0x00000010
#define LEU_FILTER_IN05       0x00000020
#define LEU_FILTER_IN06       0x00000040
#define LEU_FILTER_IN07       0x00000080
#define LEU_FILTER_IN08       0x00000100
#define LEU_FILTER_IN09       0x00000200
#define LEU_FILTER_IN10       0x00000400
#define LEU_FILTER_IN11       0x00000800
#define LEU_FILTER_IN12       0x00001000
#define LEU_FILTER_IN13       0x00002000
#define LEU_FILTER_IN14       0x00004000
#define LEU_FILTER_IN15       0x00008000
#define LEU_FILTER_OUT01      0x00020000
#define LEU_FILTER_OUT02      0x00040000
#define LEU_FILTER_OUT03      0x00080000
#define LEU_FILTER_OUT04      0x00100000
#define LEU_FILTER_OUT05      0x00200000
#define LEU_FILTER_OUT06      0x00400000
#define LEU_FILTER_OUT07      0x00800000
#define LEU_FILTER_OUT08      0x01000000
#define LEU_FILTER_OUT09      0x02000000
#define LEU_FILTER_OUT10      0x04000000
#define LEU_FILTER_OUT11      0x08000000
#define LEU_FILTER_OUT12      0x10000000
#define LEU_FILTER_OUT13      0x20000000
#define LEU_FILTER_OUT14      0x40000000
#define LEU_FILTER_OUT15      0x80000000
#define LEU_FILTER_ADDRCOUNT  128
typedef uint32_t LEU_FILTER[LEU_FILTER_ADDRCOUNT];

////////////////////////////////////////////////////////////
// Trigger
////////////////////////////////////////////////////////////
// Factor
#define LEU_TRIGGER_ERR_PID             0x00000001
#define LEU_TRIGGER_ERR_CRC             0x00000002
#define LEU_TRIGGER_BUSRESET            0x00000008
#define LEU_TRIGGER_SUSPEND             0x00000010
#define LEU_TRIGGER_DISCONNECT          0x00000020
#define LEU_TRIGGER_TOKENPACKET         0x00000040
#define LEU_TRIGGER_HANDSHAKEPACKET     0x00000080
#define LEU_TRIGGER_ADDR                0x00000100
#define LEU_TRIGGER_ENDP                0x00000200
#define LEU_TRIGGER_OVER_TRANSACTION    0x00000400
#define LEU_TRIGGER_EXTIN1H             0x00010000
#define LEU_TRIGGER_EXTIN2H             0x00020000
#define LEU_TRIGGER_EXTIN3H             0x00040000
#define LEU_TRIGGER_EXTIN4H             0x00080000
#define LEU_TRIGGER_EXTIN1L             0x00100000
#define LEU_TRIGGER_EXTIN2L             0x00200000
#define LEU_TRIGGER_EXTIN3L             0x00400000
#define LEU_TRIGGER_EXTIN4L             0x00800000
// Action
#define LEU_TRIGGER_LAST_SEQUENCE       0x0020
#define LEU_TRIGGER_MEASURE_STOP        0x0080
#define LEU_TRIGGER_EXTOUT1H            0x0100
#define LEU_TRIGGER_EXTOUT2H            0x0200
#define LEU_TRIGGER_EXTOUT3H            0x0400
#define LEU_TRIGGER_EXTOUT4H            0x0800
#define LEU_TRIGGER_EXTOUT1L            0x1000
#define LEU_TRIGGER_EXTOUT2L            0x2000
#define LEU_TRIGGER_EXTOUT3L            0x4000
#define LEU_TRIGGER_EXTOUT4L            0x8000

#define LEU_TRIGGER_SIZE_0              0x00
#define LEU_TRIGGER_SIZE_1_64           0x02
#define LEU_TRIGGER_SIZE_1_32           0x04
#define LEU_TRIGGER_SIZE_1_16           0x08
#define LEU_TRIGGER_SIZE_1_8            0x10
#define LEU_TRIGGER_SIZE_1_4            0x20
#define LEU_TRIGGER_SIZE_1_2            0x40
#define LEU_TRIGGER_SIZE_1_1            0x80

#define LEU_TRIGGER_EXTIN1_EDGE         0x01
#define LEU_TRIGGER_EXTIN2_EDGE         0x02
#define LEU_TRIGGER_EXTIN3_EDGE         0x04
#define LEU_TRIGGER_EXTIN4_EDGE         0x08
#define LEU_TRIGGER_EXTOUT1_PULSE       0x01
#define LEU_TRIGGER_EXTOUT2_PULSE       0x02
#define LEU_TRIGGER_EXTOUT3_PULSE       0x04
#define LEU_TRIGGER_EXTOUT4_PULSE       0x08
#define LEU_TRIGGER_EXTOUT1_INITIAL_HI  0x01
#define LEU_TRIGGER_EXTOUT2_INITIAL_HI  0x02
#define LEU_TRIGGER_EXTOUT3_INITIAL_HI  0x04
#define LEU_TRIGGER_EXTOUT4_INITIAL_HI  0x08

#define LEU_TRIGGER_COUNT			    8

typedef struct {
	struct {
		uint32_t factor;
		uint8_t addr;
		uint8_t endp;
		uint8_t packet_token;
		uint8_t packet_handshake;
		uint8_t data_length;
		uint8_t data[8];
		uint8_t mask[8];
		uint16_t action;
	} trig[LEU_TRIGGER_COUNT];
	uint8_t post_size;
	uint8_t setting_extin;
	uint8_t setting_extout_pulse;
	uint8_t setting_extout_initial;
} LEU_TRIGGER;

////////////////////////////////////////////////////////////
// Data structure
////////////////////////////////////////////////////////////
// status
#define LEU_DATA_STATUS_PACKET_LS		1
#define LEU_DATA_STATUS_PACKET_FS		2
#define LEU_DATA_STATUS_PACKET_HS		3
#define LEU_DATA_STATUS_PACKET_UNKNOWN	4
#define LEU_DATA_STATUS_BUSRESET		5
#define LEU_DATA_STATUS_SUSPEND			6
#define LEU_DATA_STATUS_DISCONNECT		7
#define LEU_DATA_STATUS_DEVCHIRP		8
#define LEU_DATA_STATUS_HUBCHIRP		9
#define LEU_DATA_STATUS_VBUS_LOGGER		10
#define LEU_DATA_STATUS_TRIGGER			11
#define LEU_DATA_STATUS_TRIGGER_EXT		12
#define LEU_DATA_STATUS_UNKNOWN			0xff
// pid
#define LEU_DATA_PID_RSV		0xf0
#define LEU_DATA_PID_OUT		0xe1
#define LEU_DATA_PID_ACK		0xd2
#define LEU_DATA_PID_DATA0		0xc3
#define LEU_DATA_PID_PING		0xb4
#define LEU_DATA_PID_SOF		0xa5
#define LEU_DATA_PID_NYET		0x96
#define LEU_DATA_PID_DATA2		0x87
#define LEU_DATA_PID_SPLIT		0x78
#define LEU_DATA_PID_IN			0x69
#define LEU_DATA_PID_NAK		0x5a
#define LEU_DATA_PID_DATA1		0x4b
#define LEU_DATA_PID_PRE_ERR	0x3c
#define LEU_DATA_PID_SETUP		0x2d
#define LEU_DATA_PID_STALL		0x1e
#define LEU_DATA_PID_MDATA		0x0f

typedef struct {
	uint8_t status;		// LEU_DATA_STATUS
	uint64_t timestamp;			// timestamp(nsec) -1:no timestamp
	union {
		struct {
			uint8_t		PID;
			uint16_t	length;
			uint8_t		*payload;
		} packet;
		struct {
			float		time;
		} busreset;
		struct {
			uint16_t	no;
		} trigger;
		struct {
			float	voltage;
			int16_t	current;
		} vbus;
	};
} LEU_data;

////////////////////////////////////////////////////////////
// Functions
////////////////////////////////////////////////////////////
extern void LEU_GetDllVersion(int32_t *version);
extern int LEU_Init();
extern void LEU_End();
extern void LEU_GetLastError(int32_t *error);
extern int LEU_Connect(char *serial, uint32_t buffersize);
extern int LEU_Disconnect(LEU hLEU);
extern int LEU_Measure(LEU hLEU);
extern int LEU_MeasureVBUS(LEU hLEU, int32_t cycle);
extern int LEU_Stop(LEU hLEU);
extern int LEU_Read(LEU hLEU, LEU_data *data);
extern int LEU_SetFilter(LEU hLEU, uint16_t filter);
extern int LEU_SetFilter_AddrEndp(LEU hLEU, LEU_FILTER filter);
extern int LEU_SetTrigger(LEU hLEU, LEU_TRIGGER trigger);
extern int LEU_GetState(LEU hLEU, int32_t *state);
extern int LEU_ConnectNoSerial(uint32_t buffersize);

#ifdef __cplusplus
}
#endif

#endif
