#include <stdio.h>
#include <string.h>
#include "LEU_LIB.h"

int connect(char *serial)
{
	int LE;
	printf("\n connect %s \n", serial);
	if( (LE = LEU_Connect(serial, 100*1024)) != LEU_SUCCESS ){
		printf("\n fail! \n");
		return -1;
	}
	return LE;
}

int disconnect(int LE)
{
	if( LEU_Disconnect(LE) != LEU_SUCCESS ){
		return -1;
	}
	return 0;
}

int measure(int LE)
{
	LEU_TRIGGER trg;
	memset(&trg,0,sizeof(LEU_TRIGGER));
	trg.trig[0].factor = LEU_TRIGGER_DISCONNECT;
	trg.trig[0].action = LEU_TRIGGER_MEASURE_STOP;
	trg.post_size = LEU_TRIGGER_SIZE_0;

	if( (LE = LEU_SetTrigger(LE,trg)) != LEU_SUCCESS ){
		printf("\n fail! \n");
		return -1;
	}

	if( LEU_Measure(LE) != LEU_SUCCESS ){
		return -1;
	}
	return 0;
}

int measurevbus(int LE)
{
	if( LEU_MeasureVBUS(LE,LEU_VBUS_CYCLE_100ms) != LEU_SUCCESS ){
		return -1;
	}
	return 0;
}

int stop(int LE)
{
	if( LEU_Stop(LE) != LEU_SUCCESS ){
		return -1;
	}
	return 0;
}

int read_data(int LE)
{
	int i, j;
	for( i = 0; i < 100; i ++ ){
		LEU_data data;
		if( LEU_Read(LE, &data) != 0 ){
			break;
		}
		switch(data.status){
		case LEU_DATA_STATUS_PACKET_LS		:	printf("PACKET_LS");		break;
		case LEU_DATA_STATUS_PACKET_FS		:	printf("PACKET_FS");		break;
		case LEU_DATA_STATUS_PACKET_HS		:	printf("PACKET_HS");		break;
		case LEU_DATA_STATUS_PACKET_UNKNOWN	:	printf("PACKET_UNKNOWN");	break;
		case LEU_DATA_STATUS_BUSRESET		:	printf("BUSRESET");			break;
		case LEU_DATA_STATUS_SUSPEND		:	printf("SUSPEND");			break;
		case LEU_DATA_STATUS_DISCONNECT		:	printf("DISCONNECT");		break;
		case LEU_DATA_STATUS_DEVCHIRP		:	printf("DEVCHIRP");			break;
		case LEU_DATA_STATUS_HUBCHIRP		:	printf("HUBCHIRP");			break;
		case LEU_DATA_STATUS_VBUS_LOGGER	:	printf("LOGGER");			break;
		case LEU_DATA_STATUS_TRIGGER		:	printf("TRIGGER");			break;
		case LEU_DATA_STATUS_TRIGGER_EXT	:	printf("TRIGGER_EXT");		break;
		default:
		case LEU_DATA_STATUS_UNKNOWN		:	printf("UNKNOWN");			break;
		}
		if(data.timestamp != -1){
			printf("/%014lldns", data.timestamp);
		}

		if(data.status >= LEU_DATA_STATUS_PACKET_LS && data.status <= LEU_DATA_STATUS_PACKET_UNKNOWN ){
			switch(data.packet.PID){
			case LEU_DATA_PID_RSV		:	printf("/RSV:");		break;
			case LEU_DATA_PID_OUT		:	printf("/OUT:");		break;
			case LEU_DATA_PID_ACK		:	printf("/ACK:");		break;
			case LEU_DATA_PID_DATA0		:	printf("/DATA0:");		break;
			case LEU_DATA_PID_PING		:	printf("/PING:");		break;
			case LEU_DATA_PID_SOF		:	printf("/SOF:");		break;
			case LEU_DATA_PID_NYET		:	printf("/NYET:");		break;
			case LEU_DATA_PID_DATA2		:	printf("/DATA2:");		break;
			case LEU_DATA_PID_SPLIT		:	printf("/SPLIT:");		break;
			case LEU_DATA_PID_IN		:	printf("/IN:");			break;
			case LEU_DATA_PID_NAK		:	printf("/NAK:");		break;
			case LEU_DATA_PID_DATA1		:	printf("/DATA1:");		break;
			case LEU_DATA_PID_PRE_ERR	:	printf("/PRE_ERR:");	break;
			case LEU_DATA_PID_SETUP		:	printf("/SETUP:");		break;
			case LEU_DATA_PID_STALL		:	printf("/STALL:");		break;
			case LEU_DATA_PID_MDATA		:	printf("/MDATA:");		break;
			default:						printf("/PID error:");	break;
			}
			for( j = 0; j < data.packet.length; j ++ ){
				if( j == 8 ){
					printf("...");
					break;
				}
				printf("%02x ", data.packet.payload[j]);
			}
		} else if(data.status == LEU_DATA_STATUS_VBUS_LOGGER ){
			printf("%fV %dmA", data.vbus.voltage, data.vbus.current);
		} else {
		}
		printf("\n");
	}
	return 0;
}

int main(int argc, char *argv[])
{
	int LE = -1;

	if( argc != 2 ){
		printf("Usage : sample.o <serial number>\n");
		return 0;
	}

	if( LEU_Init() != 0 ){
		printf("Init failed\n");
		return 0;
	}

	while(1){
		int result;
		int c;

		printf("-------------\n");
		printf("0:Connect\n");
		printf("1:Disconnect\n");
		printf("2:Measure\n");
		printf("3:MeasureVBUS\n");
		printf("4:Stop\n");
		printf("R:Read\n");
		printf("Q:** quit **\n");
		printf("-------------\n");
		printf("command?:\n");

		do {
			c = getchar();
		} while( c == 0x0a );

		if( c == 'q' || c == 'Q' ){
			break;
		}
		switch(c){
		case '0':	LE = result = connect(argv[1]);	break;
		case '1':	result = disconnect(LE);		break;
		case '2':	result = measure(LE);			break;
		case '3':	result = measurevbus(LE);		break;
		case '4':	result = stop(LE);				break;
		case 'r':
		case 'R':	result = read_data(LE);			break;
		}
		if( result < 0 ){
			int error;
			LEU_GetLastError(&error);
			printf("*** ERROR:%d ***\n", error);
			break;
		}
	}

	LEU_End();
	return 0;
}

